/*
 Author: Sebastien Verel, Univ. du Littoral Côte d'Opale, France
 Date: 2020/10/16, Version 1
*/

#ifndef _filter_h
#define _filter_h

#include <fstream>
#include <vector>

#include <solution.h>
#include <population.h>


using namespace std;

/*
    Compared solutions based on dominance relation: (maximisation)
     true when sol1 is dominated by sol2
     false when sol1 is not dominated by sol2
 */
bool dominated(Solution & sol1, Solution & sol2) {
    unsigned i = 0;

    bool ok = false; // flag= ok whe one objective of sol2 is strictly better

    // check fif all objective are great (max)
    while (i < sol1.objective_vector.size() && sol1.objective_vector[i] <= sol2.objective_vector[i]) {
        ok = ok || (sol1.objective_vector[i] < sol2.objective_vector[i]); // true when one objective is strictly better
        i++;
    }

    return (i == sol1.objective_vector.size() && ok);
}

/*
    Filter dominated solution from fin
*/
void filter_dominated(Population & fin, Population & fout) {
    unsigned j;

    // for all solutions in the set
    for(unsigned i = 0; i < fin.size(); i++) {
        j = 0; 

        // while solution j non domintes the solution i
        while(j < fin.size() && not dominated(fin[i], fin[j])) {
            j++;
            if (j == i) j++; // skip the solution i itself
        }

        // when all solutions not dominates solution i
        if (j >= fin.size()) {
            fout.push_back(fin[i]);
        }
    }

}

/*
    Add one solution to the population of mutually non dominated solutions
    So, update the population according to dominance relation
*/
void filter_addOne(Population & pop, Solution & solution) {
    // TO DO
}

#endif