/*
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; version 3 
    of the License.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Contact: http://mocobench.sourceforge.net

Authors:
    Arnaud Liefooghe <arnaud.liefooghe@lifl.fr>
    Sebastien Verel  <sebastien.verel@inria.fr>
*/

/*
  
  mkdir build
  cd build
  cmake ..
  make
  ./random_solutions ../instances/instance.dat

 */

#include <iostream>
#include <vector>
#include <string>
#include <time.h>
#include <random>

#include <solution.h>
#include <mubqpEval.h>
#include <rnd.h>
#include <population.h>
#include <filter.h>
#include <weightedFitness.h>

/***************************************************************/
/*                 Main                                        */
/***************************************************************/

int main(int argc, char *argv[]) {
  //-----------------------------------------------
  // evaluation function of the rMNK-landscapes

  if (argc != 4) {
    std::cerr << "Invalid number of parameters. Use the command line: ./random_solutions instance.dat popSize seed" << std::endl;
    exit(1);
  }

  MUBQPEval mubqp(argv[1]);

  // size of the bit string
  unsigned N = mubqp.getN();
  // objective space dimension
  unsigned M = mubqp.getM();

  // population size
  unsigned popSize = atoi(argv[2]);

  // weighted fitness value
  std::vector<double> weights = { 0.5, 0.5 };

  WeightedFitness scalar_eval(weights);

  // seed of the pseudo-random generator
  long unsigned int seed = atoi(argv[3]);

  // random generator
  std::mt19937 rnd_gen;
  rnd_gen.seed(seed);

  //-----------------------------------------------
  // initialization of the population

  // random solution generator
  RandomSolution rnd(rnd_gen, N);

  // population of solutions
  Population pop;

  // Dummy solution
  Solution solution;

  for(unsigned i = 0; i < popSize; i++) {
    // generate random solution
    rnd(solution);

    // evaluation
    mubqp.eval(solution);

    // scalar value
    scalar_eval(solution);

    // add to the population
    pop.push_back(solution);
  }

  //-----------------------------------------------
  // ouput : print the population


  std::cout << pop << std::endl;

  //-----------------------------------------------
  // filter Population

  Population non_dominated;

  filter_dominated(pop, non_dominated);

  //-----------------------------------------------
  // ouput : print the solution and its objective vector

  std::cout << non_dominated << std::endl;

  return 0;
}

